"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const events_1 = require("events");
const types_1 = require("../plugins/types");
const build_1 = require("./build");
/**
 * A sequence of CodeEngine plugins that can be used to run builds.
 * @internal
 */
class BuildPipeline extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        this._plugins = [];
    }
    /**
     * The number of plugins in the build pipeline.
     */
    get size() {
        return this._plugins.length;
    }
    /**
     * Adds a plugin to the build pipeline
     */
    add(plugin) {
        this._plugins.push(plugin);
    }
    /**
     * Deletes the contents of the destination(s).
     */
    async clean(context) {
        let cleaners = this._plugins.filter(types_1.hasClean);
        await Promise.all(cleaners.map((plugin) => plugin.clean(context)));
    }
    /**
     * Runs a full build of all source files.
     */
    async build(concurrency, context) {
        let buildContext = {
            ...context,
            fullBuild: true,
            partialBuild: false,
            changedFiles: [],
        };
        this._emitBuildStarting(buildContext);
        let files = readAllSources(this._plugins, buildContext);
        let steps = this._plugins.filter(types_1.isBuildStep);
        let summary = await build_1.runBuild(files, steps, concurrency, buildContext);
        this._emitBuildFinished(buildContext, summary);
        return summary;
    }
    /**
     * Watches source files for changes and runs incremental re-builds whenever changes are detected.
     */
    watch(delay, concurrency, context) {
        let fileChanges = watchAllSources(this._plugins, context);
        let batchedFileChanges = utils_1.debounceIterable(fileChanges, delay);
        Promise.resolve()
            .then(async () => {
            for await (let changedFiles of batchedFileChanges) {
                let buildContext = {
                    ...context,
                    fullBuild: false,
                    partialBuild: true,
                    changedFiles,
                };
                this._emitBuildStarting(buildContext);
                // Only build the files that still exist (i.e. not the ones that were deleted)
                let files = changedFiles.filter((file) => file.change !== "deleted" /* Deleted */);
                let steps = this._plugins.filter(types_1.isBuildStep);
                let summary = await build_1.runBuild(utils_1.iterate(files), steps, concurrency, buildContext);
                this._emitBuildFinished(buildContext, summary);
            }
        })
            .catch(this.emit.bind(this, "error" /* Error */));
    }
    /**
     * Removes all plugins and stops watching source files.
     */
    async dispose(context) {
        let plugins = this._plugins;
        this._plugins = [];
        let needDisposed = plugins.filter(types_1.hasDispose);
        await Promise.all(needDisposed.map((plugin) => plugin.dispose(context)));
    }
    /**
     * Emits a "buildStarting" event
     */
    _emitBuildStarting(context) {
        if (this.listenerCount("buildStarting" /* BuildStarting */) > 0) {
            context = { ...context };
            context.changedFiles = context.changedFiles.slice();
            this.emit("buildStarting" /* BuildStarting */, context);
        }
    }
    /**
     * Emits a "buildFinished" event
     */
    _emitBuildFinished(context, summary) {
        if (this.listenerCount("buildFinished" /* BuildFinished */) > 0) {
            context = { ...context, ...summary };
            context.changedFiles = context.changedFiles.slice();
            this.emit("buildFinished" /* BuildFinished */, context);
        }
    }
}
exports.BuildPipeline = BuildPipeline;
/**
 * Reads all source files simultaneously from all file sources.
 */
function readAllSources(plugins, context) {
    let sources = plugins.filter(types_1.isFileSource);
    let fileGenerators = sources.map((source) => source.read(context));
    return utils_1.joinIterables(...fileGenerators);
}
/**
 * Watches all source files for changes.
 */
function watchAllSources(plugins, context) {
    let watchers = plugins.filter(types_1.hasWatch);
    let fileGenerators = watchers.map((watcher) => watcher.watch(context));
    return utils_1.joinIterables(...fileGenerators);
}
//# sourceMappingURL=build-pipeline.js.map